﻿using System;
using System.IO;
using System.Windows.Forms;

namespace RCM {
    public partial class SQLiteControl : UserControl {
        private SQLite _sqLite;

        public SQLiteControl(SQLite sqLite) {
            InitializeComponent();
            _sqLite = sqLite;
            textBoxDatabase.Text = _sqLite.FileName;
            textBoxEncPassword.Text = _sqLite.AESPassword;
        }

        private void ButtonSelectDatabase_Click(object sender, EventArgs e) {
            if (saveFileDialog.ShowDialog() == DialogResult.OK) {
                textBoxDatabase.Text = saveFileDialog.FileName;
            }
        }

        private void ButtonTestConnection_Click(object sender, EventArgs e) {
            try {
                string version = _sqLite.TestConnection(textBoxDatabase.Text);
                MessageBox.Show(version, "Test successful", MessageBoxButtons.OK, MessageBoxIcon.Information);
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "Test failed", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonCreateSchema_Click(object sender, EventArgs e) {
            if (File.Exists(textBoxDatabase.Text)) {
                if (MessageBox.Show("Do you want to delete existing database and create a new one?", "Database already exists", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) != DialogResult.Yes) {
                    return;
                }
                File.Delete(textBoxDatabase.Text);
            }
            _sqLite.CreateSchema(textBoxDatabase.Text);
            MessageBox.Show("New database successfully created", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void ButtonUnmaskEncPassword_Click(object sender, EventArgs e) {
            textBoxEncPassword.UseSystemPasswordChar = !textBoxEncPassword.UseSystemPasswordChar;
        }
    }
}
